﻿'In response to requests to show how the binary data stream returned by DATAQ Instruments'
'starter kits can be decoded in real time, we offer this program. 

'The program allows you to select a COM port to which is connected a DATAQ Instruments Starter kit.
'It accepts a typed protocol-level command and displays the response in the output window.
'It assumes that the scan list for the given starter kit is loaded ONLY with analog channels.
'Feel free to experiment with other channel types with your own code.
'The program outputs ASCII data directly if the SK supports and is programmed for that mode,
'or convert binary  data into equivalent ADC counts in ASCII before outputting. 
'The program assumes that the operator uses and understands protocol-level commands for 
'the instrument-under-test. Protocol docs for each starer kit can be found at these links:

'DI-145, http://www.dataq.com/resources/pdfs/misc/di-145-protocol1_07.pdf
'DI-149, http://www.dataq.com/resources/pdfs/misc/di-149-protocol.pdf
'DI-155, http://www.dataq.com/resources/pdfs/misc/di-155-protocol.pdf
'DI-245, http://www.dataq.com/resources/pdfs/support_articles/DI-245-protocol.pdf

'Report bugs to http://www.dataq.com/support/ticket/ticket-form.html



Public Class frmProtocolTool
    Protected Friend WithEvents DataqSerialPort As System.IO.Ports.SerialPort
    Protected WithEvents PortTimeOut As New Stopwatch
    Dim InitialByteCount As Short 'command echo character counter
    Dim AsciiData As Boolean ' flags data output format from the instrument as ASCII 
    Dim CurrentlyScanningBinary As Boolean 'flags when scanning binary so ascii commends echo properly
    Dim ModelNumber As String 'holds the model number of the instrument in use
    Dim PurgeBuffer As Boolean = False 'flags the need to gracefully stop a binary scanning process
    Public ByteBuffer(1000) As Byte 'holds raw data delivered by the instrument via the COM port


    Private Sub frmProtocolTool_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        DataqSerialPort = New System.IO.Ports.SerialPort() 'create a new serial port
        cmbPorts.Items.AddRange(IO.Ports.SerialPort.GetPortNames) 'populate active COM port names in the drop-down menu
        lbOutput.Items.Clear() 'clear the display 
    End Sub
    Private Sub Connect()

        'need to close the current active port before opening another
        If DataqSerialPort.IsOpen Then
            DataqSerialPort.Close()
            While ((DataqSerialPort.IsOpen) And (PortTimeOut.ElapsedMilliseconds < 1000))
                'wait for either the port to close or for one second to lapse
            End While
        End If

        'open the new port
        DataqSerialPort.PortName = cmbPorts.SelectedItem.ToString
        lblConnectStatus.Text = "Connected"
        btnSendCommand.Enabled = True

        'finally open the new port
        DataqSerialPort.Open()
        PortTimeOut.Start()
        While ((DataqSerialPort.IsOpen = False) And (PortTimeOut.ElapsedMilliseconds < 1000))

        End While
    End Sub

    Private Sub btnSendCommand_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSendCommand.Click
        Dim TextToSend As String = ""

        Select Case True
            Case txtCommand.Text = "asc"
                If ModelNumber <> "DI-245" Then
                    '(the DI-245 does not support the "asc" command.)
                    MessageBox.Show("ADC values may not display properly at faster sampling rates when the output mode is ASCII.", _
                                    "ASCII Mode Data Output", MessageBoxButtons.OK, MessageBoxIcon.Warning)
                    AsciiData = True
                End If

            Case txtCommand.Text = "bin"
                AsciiData = False
        End Select

        Select Case True
            Case (txtCommand.Text = "S1") Or (txtCommand.Text = "start")
                PurgeBuffer = False 'reset PurgeBuffer
                TextToSend = Chr(0) + "S1" 'use the universal start command "S1" preceded by a NULL character
                InitialByteCount = 0 'we're beginning a scan, so reset the initial byte count
                If Not AsciiData Then
                    CurrentlyScanningBinary = True
                End If

            Case (txtCommand.Text = "S0") Or (txtCommand.Text = "stop")
                TextToSend = Chr(0) + "S0" 'Use the universal stop command "S0" preceded by a NULL character
                If CurrentlyScanningBinary Then
                    PurgeBuffer = True 'we're scanning binary and trying to stop gracefully. Set PurgeBuffer to flag this state
                End If
     
            Case Else
                TextToSend = txtCommand.Text 'otherwise, just send the typed command
                PurgeBuffer = False
        End Select
        'send the command
        DataqSerialPort.Write(TextToSend & vbCr)
        txtCommand.Focus()
        txtCommand.Clear()
    End Sub

    'Cannot access lbOutput from inside the event handler(ReceiveComPortData) without delegating, so...
    Private Delegate Sub UpdateListDelegate(ByVal itemName As String) 'create new class UpdateListDelegate
    Private Sub UpdateList(ByVal itemName As String)
        If Me.InvokeRequired Then
            Me.Invoke(New UpdateListDelegate(AddressOf UpdateList), itemName) 'invoke delegate then...
        Else
            lbOutput.Items.Add(itemName) '...output string to list box
        End If
    End Sub

    Private Sub ReceiveComPortData _
        (ByVal sender As Object, ByVal e As IO.Ports.SerialDataReceivedEventArgs) Handles DataqSerialPort.DataReceived
        'this is the workhorse, event-driven sub that's active while data is xmitted from the serial port. If scanning,
        'that's almost all the time. The sub is intimidatingly long, but that's misleading. It needs to deal with three
        'possible binary decoding schemes depending upon the connected starter kit model. That expands length by a
        'almost a factor of three. We could have condensed this into one decoder to keep the sub shorter, but that would 
        'sacrifice clarity. Afterall, the priamry goal of this program is to show how binary streams returned by 
        'starter kits can be decoded and used. 

        Dim BytesToRead As Integer = DataqSerialPort.BytesToRead

        'defining the following as STATIC, since the code could overrun ByteBuffer and exit this sub before 
        'a complete conversion of returned values, especially at slower sampling rates
        Static HighByte As Short 'used to construct ADC counts if in the binary output mode
        Static LowByte As Short 'used to construct ADC counts if in the binary output mode
        Static LowByteFlag As Boolean 'used to construct ADC counts if in the binary output mode
        Static ResponseString As String 'holds the response string, which is constructed over multiple passes of
        'the sub's FOR statement

        ReDim ByteBuffer(BytesToRead - 1) 'holds instrument's byte-by-byte response delivered via the serial port

        'fill ByteBuffer with byte data returned by the instrument via the serial port
        DataqSerialPort.Read(ByteBuffer, 0, BytesToRead)

        For Each ReceivedByte In ByteBuffer

            'This code block synchronizes the beginning of a binary data scan.
            'when starting a binary scan, strip the response string of 'S1', which produces garbage in the output.
            'Exit the loop on the low byte of the first channel in the scan list, and only after at least two bytes 
            'have been received.
            While (InitialByteCount < 2) And (ReceivedByte And 1) <> 0 And CurrentlyScanningBinary
                InitialByteCount = InitialByteCount + 1
                ResponseString = "" 'we're starting anew, so null ResponseString of any previous scan data.
                Continue For
            End While

            'This code block synchronizes the end of a binary data scan
            If PurgeBuffer Then
                'get here if the STOP command was issued during a binary scanning operation.
                'the following code ensures that the process stops cleanly AFTER the last channel in the scan list is displayed.
                If (ReceivedByte And 1) = 0 Then
                    'get here if this is the first byte of a new scan, meaning that the previous byte
                    'was the last value for the last item in the scan list
                    CurrentlyScanningBinary = False 'lower the binary scanning flag
                    ResponseString = "" 'purge
                    Exit Sub 'we're done. Nothing to do but exit
                End If
                'get here if not the first byte of a new scan, so continue outputting values for now. 
            End If

            'The remainder of the sub outputs ascii data, or binary data converted to ascii 
            'depending upon the command and/or mode of operation
            Select Case True
                Case AsciiData Or Not (CurrentlyScanningBinary)
                    'get here if expecting an ascii data or command response output
                    Select Case True
                        Case ReceivedByte = 13
                            'get here if character is a carriage return
                            UpdateList(ResponseString) ' we're done with this line...display it
                            Select Case True
                                'Since these strings terminate with <cr> 
                                'detect the model we're working with so we know how to decode binary data
                                Case ResponseString = "info 1 1550"
                                    ModelNumber = "DI-155"
                                Case ResponseString = "info 1 1450"
                                    ModelNumber = "DI-145"
                                Case ResponseString = "info 1 1490"
                                    ModelNumber = "DI-149"
                                Case ResponseString = "info 1 2450"
                                    ModelNumber = "DI-245"
                            End Select
                            ResponseString = "" 'null ResponseString

                        Case Else
                            'get here if the character is not <cr>. If not, simply append the ASCII character
                            'to ResponseString
                            ResponseString = ResponseString + CChar(ChrW(ReceivedByte)) 'append item to string
                    End Select

                Case Else
                    'get here when expecting binary ADC data values

                    'since LowByteFlag is false when the LSB of the lowest byte of the first scan list item = 0, 
                    'force it now. Synchronization errors can otherwise occur. 
                    If (ReceivedByte And 1) = 0 Then
                        LowByteFlag = False
                    End If

                    Select Case True
                        Case (ModelNumber = "DI-155") Or (ModelNumber = "DI-245")
                            'get here if decoding binary data stream from a DI-155 or DI-245
                            'assumes only analog channels are enabled. Use CONTINUE FOR to save time, 
                            'since any further tests waste it after a byte is properly decoded.
                            Select Case True
                                Case ((ReceivedByte And 1) = 0) And (Not LowByteFlag)
                                    'get here on the first byte of channel that's first in the scan list
                                    UpdateList(ResponseString) 'print the previous scan
                                    ResponseString = "" 'clear response string
                                    LowByte = ReceivedByte >> 1 'strip sync bit and copy to LowByte
                                    LowByteFlag = True 'we've detected a low byte of an acquired channel
                                    Continue For 'done with this byte, so otta here

                                Case LowByteFlag
                                    'get here on the second byte of any channel value
                                    HighByte = ReceivedByte >> 1 'strip sync bit
                                    HighByte = HighByte << 7 'move this byte into the most significant position
                                    HighByte = HighByte Or LowByte 'HighByte becomes the complete ADC word
                                    HighByte = HighByte Xor 8192 'invert the MSB of the ADC value
                                    If (HighByte And 8192) = 0 Then
                                        HighByte = HighByte And 8191 'we're working with a positive value so sign-extend as zeros
                                    Else
                                        HighByte = HighByte Or -8192 'we're working with a negative value so sign-extend as 1's
                                    End If
                                    ResponseString = ResponseString + HighByte.ToString
                                    LowByteFlag = False 'Finished decoding the low byte, so clear the flag
                                    Continue For 'done with this byte, so otta here

                                Case Not (LowByteFlag)
                                    'get here on the first byte of any item that's second or higher in the scan list
                                    ReceivedByte = ReceivedByte >> 1 'strip sync bit
                                    LowByte = 0
                                    LowByte = LowByte Or ReceivedByte 'and copy to LowByte
                                    LowByteFlag = True 'we've detected the low byte of an acquired channel
                                    ResponseString = ResponseString + ", " 'we have another value to convert so insert a comma
                                    Continue For 'done with this byte, so otta here
                            End Select

                        Case ModelNumber = "DI-145"
                            'get here if converting binary data from a DI-145
                            'assumes that only analog channels are enabled. Use CONTINUE FOR to save time, 
                            'since any further tests waste it after a byte is properly decoded.
                            Select Case True
                                Case ((ReceivedByte And 1) = 0) And (Not LowByteFlag)
                                    'get here on the first byte of channel that's first in the scan list
                                    UpdateList(ResponseString) 'print the previous scan
                                    ResponseString = "" 'clear response string
                                    LowByte = ReceivedByte >> 3 'strip sync bit and two digital bits
                                    LowByteFlag = True 'we've detected a low byte of an acquired channel
                                    Continue For 'done with this byte, so otta here

                                Case LowByteFlag
                                    'get here on the second byte of any channel value
                                    HighByte = ReceivedByte >> 1 'strip sync bit
                                    HighByte = HighByte << 5 'move this byte into the most significant position
                                    HighByte = HighByte Or LowByte 'HighByte becomes the complete ADC word
                                    HighByte = HighByte Xor 2048 'invert the MSB of the ADC value
                                    If (HighByte And 2048) = 0 Then
                                        HighByte = HighByte And 2047 'we're working with a positive value so sign-extend as zeros
                                    Else
                                        HighByte = HighByte Or -2048 'we're working with a negative value so sign-extend as 1's
                                    End If
                                    ResponseString = ResponseString + HighByte.ToString
                                    LowByteFlag = False 'Finished decoding the low byte, so clear the flag
                                    Continue For 'done with this byte, so otta here

                                Case Not (LowByteFlag)
                                    'get here on the first byte of any item that's second or higher in the scan list
                                    ReceivedByte = ReceivedByte >> 3 'strip sync bit and two digital bits
                                    LowByte = 0
                                    LowByte = LowByte Or ReceivedByte 'and copy to LowByte
                                    LowByteFlag = True 'we've detected the low byte of an acquired channel other than the first
                                    ResponseString = ResponseString + ", " 'we have another value to convert so insert a comma
                                    Continue For 'done with this byte, so otta here
                            End Select

                        Case ModelNumber = "DI-149"
                            'get here if decoding binary data stream from a DI-149
                            'assumes that only analog channels are enabled. Use CONTINUE FOR to save time, 
                            'since any further tests waste it after a byte is properly decoded.
                            Select Case True
                                Case ((ReceivedByte And 1) = 0) And (Not LowByteFlag)
                                    'get here on the first byte of channel that's first in the scan list
                                    UpdateList(ResponseString) 'print the previous scan
                                    ResponseString = "" 'clear response string
                                    ReceivedByte = ReceivedByte >> 3 'strip sync bit and two digital bits
                                    LowByte = 0
                                    LowByte = LowByte Or ReceivedByte 'and copy to LowByte
                                    LowByteFlag = True 'we've detected a low byte of an acquired channel
                                    Continue For 'done with this byte, so otta here

                                Case LowByteFlag
                                    'get here on the second byte of any channel value
                                    HighByte = ReceivedByte >> 1 'strip sync bit
                                    HighByte = HighByte << 5 'move this byte into the most significant position
                                    HighByte = HighByte Or LowByte 'HighByte becomes the complete ADC word
                                    HighByte = HighByte Xor 2048 'invert the MSB of the ADC value
                                    If (HighByte And 2048) = 0 Then
                                        HighByte = HighByte And 2047 'we're working with a positive value so sign-extend as zeros
                                    Else
                                        HighByte = HighByte Or -2048 'we're working with a negative value so sign-extend as 1's
                                    End If
                                    ResponseString = ResponseString + HighByte.ToString
                                    LowByteFlag = False 'Finished decoding the low byte, so clear the flag
                                    Continue For 'done with this byte, so otta here

                                Case Not (LowByteFlag)
                                    'get here on the first byte of any item that's second or higher in the scan list
                                    ReceivedByte = ReceivedByte >> 3 'strip sync bit and two digital bits
                                    LowByte = 0
                                    LowByte = LowByte Or ReceivedByte 'and copy to LowByte
                                    LowByteFlag = True 'we've detected the low byte of an acquired channel other than the first
                                    ResponseString = ResponseString + ", " 'we have another value to convert so insert a comma
                                    Continue For 'done with this byte, so otta here

                            End Select
                    End Select
            End Select
        Next
        If (ResponseString <> "") And Not CurrentlyScanningBinary Then
            UpdateList(ResponseString + vbCrLf) 'this handles any command responses that don't terminate with a cr. Examples
            'are commands "A0" and "A1"
            ResponseString = "" 'null ResponseString
        End If
    End Sub
    Private Sub cmbPorts_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbPorts.SelectedIndexChanged
        Connect()
        DataqSerialPort.BaudRate = 115200 'required only for a DI-245. Ignored for other starter kits
        DataqSerialPort.DataBits = 8 'required only for a DI-245. Ignored for other starter kits
        DataqSerialPort.StopBits = 1 'required only for a DI-245. Ignored for other starter kits
        DataqSerialPort.Parity = IO.Ports.Parity.None 'required only for a DI-245. Ignored for other starter kits

        'make binary output format the default upon connection
        DataqSerialPort.Write("bin" & vbCr) 'program binary output as default (ignored by DI-245, which only supports binary)
        AsciiData = False
        'send the model number command 'info 1'. Response will be captured by the ReceiveComPortData
        'sub where model number will be noted for subsequent binary ADC decoding methods 
        DataqSerialPort.Write("info 1" & vbCr)
    End Sub

    Private Sub btnClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClear.Click
        lbOutput.Items.Clear()
    End Sub
End Class

